#!/bin/sh
# *******************************************************************************
#  Licensed Materials - Property of IBM
#  (c) Copyright IBM Corporation 2006, 2011. All Rights Reserved.
#  
#  Note to U.S. Government Users Restricted Rights:  
#  Use, duplication or disclosure restricted by GSA ADP Schedule 
#  Contract with IBM Corp. 
#  *******************************************************************************


# ******************
# * Initialization *
# ******************

START_DIR=`pwd`

# Ensure the app_upgrade script was not called directly.
if [ "$SCRIPT_DIR" = "" ]; then
	INVALID_LAUNCH=true
	SCRIPT_DIR=`dirname $0`
else
	INVALID_LAUNCH=false
fi

# Read functions
. $SCRIPT_DIR/functions.sh
getAbsolutePath $SCRIPT_DIR
SCRIPT_DIR=$ABSOLUTE_PATH

# Messages script
. $SCRIPT_DIR/locale.sh
. $SCRIPT_DIR/messages/messages$LANG_SUFFIX.sh

# Exit in case of invalid launch
if [ $INVALID_LAUNCH = true ]; then
	messages INVALID_LAUNCH
	exit 10
fi

# First argument is the kind of upgrading
# Should be only -jts, -rtc, -rqm or -rrc
if [ $# -eq 0 ]; then
	messages MISSING_APP_TYPE
	exit 10
fi
case $1 in
	-jts | -ccm | -qm | -rm)
		UPGRADED_APP=`echo $1 | cut -c2-`
		shift
		;;
	*)
		messages INVALID_APP_TYPE $1
		exit 10
		;;
esac

# Set parameters values from file
. $SCRIPT_DIR/params/${UPGRADED_APP}_params$LANG_SUFFIX.sh

# Starting dir should be server dir
getAbsolutePath $SCRIPT_DIR
SERVER_DIR=`dirname $ABSOLUTE_PATH`
if [ $START_DIR != $SERVER_DIR ]; then
	if [ "$1" = "-help" ]; then
		help
	else
		messages INVALID_START_DIR ${UPGRADED_APP}_upgrade.sh $SERVER_DIR
	fi
	exit 10
fi

# *********************
# * Arguments reading *
# *********************
while [ $# -gt 0 ]
do
	case $1 in
		-oldApplicationHome | -oldapplicationhome )
			if [ $UPGRADED_APP = jts ]; then
				messages INVALID_PARAM $1
				exit 1
			else
				if [ $# -eq 1 ]; then
					messages MISSING_PARAM_VALUE $1
					exit 1
				fi
				if [ -d $2 ]; then
					if [ "`echo $2 | cut -d'/' -f1`" = "" ]; then
						OLD_APPLICATION_HOME=$2
						shift
					else
						messages RELATIVE_PARAM_DIR $1 $2
						exit 1
					fi
				else
					messages INVALID_PARAM_DIR $1 $2
					exit 1
				fi
			fi
			;;
		-applicationContextRoot | -applicationcontextroot)
			if [ $UPGRADED_APP = jts ]; then
				messages INVALID_PARAM $1
				exit 1
			else
				if [ $# -eq 1 ]; then
					messages MISSING_PARAM_VALUE $1
					exit 1;
				fi
				APPLICATION_CONTEXT_ROOT=$2
				shift
			fi
			;;
		-newApplicationHome | -newapplicationhome)
			if [ $UPGRADED_APP = jts ]; then
				messages INVALID_PARAM $1
				exit 1
			else
				if [ $# -eq 1 ]; then
					messages MISSING_PARAM_VALUE $1
					exit 1
				fi
				if [ -d $2 ]; then
					getAbsolutePath $2
					NEW_APPLICATION_HOME=$ABSOLUTE_PATH
					shift
				else
					messages INVALID_PARAM_DIR $1 $2
					exit 1
				fi
			fi
			;;
		-oldJTSHome | -oldjtshome)
			if [ $UPGRADED_APP = jts ] || [ $UPGRADED_APP = rm ]; then
				if [ $# -eq 1 ]; then
					messages MISSING_PARAM_VALUE $1
					exit 1
				fi
				if [ -d $2 ]; then
					if [ "`echo $2 | cut -d'/' -f1`" = "" ]; then
						OLD_JTS_HOME=$2
						shift
					else
						messages RELATIVE_PARAM_DIR $1 $2
						exit 1
					fi
				else
					messages INVALID_PARAM_DIR $1 $2
					exit 1
				fi
			else
				messages INVALID_PARAM $1
				exit 1
			fi
			;;
		-updateTomcatFiles | -updatetomcatfiles)
			if [ $# -eq 1 ]; then
				messages MISSING_PARAM_VALUE $1
				exit 1
			fi
			if [ "$2" != "yes" ] && [ "$2" != "no" ]; then
				messages INVALID_PARAM_VALUE $1 $2
				exit 1
			fi
			UPDATE_TOMCAT_FILES=$2
			shift
			;;
		-oldJTSContextRoot | -oldjtscontextroot)
			if [ $UPGRADED_APP = rm ]; then
				if [ $# -eq 1 ]; then
					messages MISSING_PARAM_VALUE $1
					exit 1
				fi
				OLD_JTS_CONTEXT_ROOT=$2
				shift
			else
				messages INVALID_PARAM $1
				exit 1
			fi
			;;
		-oldApplicationTomcatHome | -oldapplicationtomcathome)
			if [ $UPGRADED_APP = rm ]; then
				if [ $# -eq 1 ]; then
					messages MISSING_PARAM_VALUE $1
					exit 1
				fi
				if [ "`echo $2 | cut -d'/' -f1`" = "" ]; then
					OLD_TOMCAT_HOME=$2
					shift
				else
					messages RELATIVE_PARAM_DIR $1 $2
					exit 1
				fi
				shift
			else
				messages INVALID_PARAM $1
				exit 1
			fi
			;;
		-oldJTSTomcatHome | -oldjtstomcathome)
			if [ $UPGRADED_APP = rm ]; then
				if [ $# -eq 1 ]; then
					messages MISSING_PARAM_VALUE $1
					exit 1
				fi
				if [ "`echo $2 | cut -d'/' -f1`" = "" ]; then
					OLD_JTS_TOMCAT_HOME=$2
					shift
				else
					messages RELATIVE_PARAM_DIR $1 $2
					exit 1
				fi
				shift
			else
				messages INVALID_PARAM $1
				exit 1
			fi
			;;
		-newJTSContextRoot | -newjtscontextroot)
			if [ $UPGRADED_APP = rm ]; then
				if [ $# -eq 1 ]; then
					messages MISSING_PARAM_VALUE $1
					exit 1
				fi
				JTS_CONTEXT_ROOT=$2
				shift
			else
				messages INVALID_PARAM $1
				exit 1
			fi
			;;
		-jtsURL | -jtsurl)
			if [ $UPGRADED_APP = rm ]; then
				if [ $# -eq 1 ]; then
					messages MISSING_PARAM_VALUE $1
					exit 1
				fi
				JTS_PUBLIC_URL=$2
				shift
			else
				messages INVALID_PARAM $1
				exit 1
			fi
			;;
		-oldTomcatHome | -oldtomcathome)
			if [ $UPGRADED_APP != rm ]; then
				if [ $# -eq 1 ]; then
					messages MISSING_PARAM_VALUE $1
					exit 1
				fi
				if [ "`echo $2 | cut -d'/' -f1`" = "" ]; then
					OLD_TOMCAT_HOME=$2
					shift
				else
					messages RELATIVE_PARAM_DIR $1 $2
					exit 1
				fi
				shift
			else
				messages INVALID_PARAM $1
				exit 1
			fi
			;;
		-jtsContextRoot | -jtscontextroot)
			if [ $UPGRADED_APP != rm ]; then
				if [ $# -eq 1 ]; then
					messages MISSING_PARAM_VALUE $1
					exit 1
				fi
				JTS_CONTEXT_ROOT=$2
				shift
			else
				messages INVALID_PARAM $1
				exit 1
			fi
			;;
		-newTomcatHome | -newtomcathome)
			if [ $UPGRADED_APP = jts ]; then
				if [ $# -eq 1 ]; then
					messages MISSING_PARAM_VALUE $1
					exit 1
				fi
				getAbsolutePath $2
				NEW_TOMCAT_HOME=$ABSOLUTE_PATH
				shift
			else
				messages INVALID_PARAM $1
				exit 1
			fi
			;;
		-newApplicationTomcatHome | -newapplicationtomcathome)
			if [ $UPGRADED_APP != jts ]; then
				if [ $# -eq 1 ]; then
					messages MISSING_PARAM_VALUE $1
					exit 1
				fi
				getAbsolutePath $2
				NEW_TOMCAT_HOME=$ABSOLUTE_PATH
				shift
			else
				messages INVALID_PARAM $1
				exit 1
			fi
			;;
		-newJTSTomcatHome | -newjtstomcathome)
			if [ $UPGRADED_APP != rm ]; then
				if [ $# -eq 1 ]; then
					messages MISSING_PARAM_VALUE $1
					exit 1
				fi
				getAbsolutePath $2
				NEW_JTS_TOMCAT_HOME=$ABSOLUTE_PATH
				shift
			else
				messages INVALID_PARAM $1
				exit 1
			fi
			;;
		-newJTSHome | -newjtshome)
			if [ $# -eq 1 ]; then
				messages MISSING_PARAM_VALUE $1
				exit 1
			fi
			if [ -d $2 ]; then
				getAbsolutePath $2
				NEW_JTS_HOME=$ABSOLUTE_PATH
				shift
			else
				messages INVALID_PARAM_DIR $1 $2
				exit 1
			fi
			;;
		-oldApplicationVersion | -oldapplicationversion)
			if [ $UPGRADED_APP = ccm ]; then
				if [ $# -eq 1 ]; then
					messages MISSING_PARAM_VALUE $1
					exit 1
				fi
				OLD_APPLICATION_VERSION=$2
				shift
			else
				messages INVALID_PARAM $1
				exit 1
			fi
			;;
		-oldJTSVersion | -oldjtsversion)
			if [ $UPGRADED_APP = jts ]; then
				if [ $# -eq 1 ]; then
					messages MISSING_PARAM_VALUE $1
					exit 1
				fi
				OLD_JTS_VERSION=$2
				shift
			else
				messages INVALID_PARAM $1
				exit 1
			fi
			;;
		-createJTSTables | -createjtstables)
			if [ $UPGRADED_APP = jts ]; then
				messages INVALID_PARAM $1
				exit 1
			else
				CREATE_JTS_TABLES=createTables
			fi
			;;
		-ignoreJTSVersionCheck | -ignorejtsversioncheck)
			if [ $UPGRADED_APP = jts ]; then
				messages INVALID_PARAM $1
				exit 1
			else
				IGNORE_JTS_VERSION_CHECK=ignoreJTSVersionCheck
			fi
			;;
		-noPrompt | -noprompt)
			NO_PROMPT=noPrompt
			;;
		-noVerify | -noverify)
			NO_VERIFY=false
			;;
		-overwrite)
			OVERWRITE=overwrite=yes
			;;
		-importFromFile | -)
			if [ $# -eq 1 ]; then
				messages MISSING_PARAM_VALUE $1
				exit 1
			fi
			if [ -f $2 ]; then
				getAbsolutePath $2
				IMPORT_FROM_FILE=$ABSOLUTE_PATH
				shift
			else
				messages INVALID_PARAM_FILE $1 $2
				exit 1
			fi
			;;
		-noStepPrompt | -nostepprompt)
			NO_STEP_PROMPT=true
			;;
		-help)
			help
			exit 2
			;;
		*)
			messages INVALID_PARAM $1
			exit 1
			;;
	esac
	shift
done


# ************************
# * Parameters verifying *
# ************************
#
# RTC parameters rules
# ====================
# 1) 2.x to 3.x upgrade
# ---------------------
#  -oldApplicationHome: mandatory, no default
#  -applicationRoot:    optional, default=jazz
#  -oldJTSHome:         --- (forbidden)
#  -jtsContextRoot:     optional, default=jts
#  -updateType:         optional, default=JAZZ_AND_TOMCAT
#  -oldTomcatHome:      optional, default=tomcat
#  -newApplicationHome: mandatory, no default
#  -newJtsHome:         optional, default=newAppHome
#  -newTomcatHome:      optional, default=tomcat
#
# 2) 3.0 to 3.x upgrade
# ---------------------
#  -oldApplicationHome: mandatory, no default
#  -applicationRoot:    optional, default=jazz
#  -oldJTSHome:         mandatory, no default
#  -jtsContextRoot:     optional, default=jts
#  -updateType:         optional, default=JAZZ_AND_TOMCAT
#  -oldTomcatHome:      optional, default=tomcat
#  -newApplicationHome: mandatory, no default
#  -newJtsHome:         optional, default=newAppHome
#  -newTomcatHome:      optional, default=tomcat

#$SCRIPT_DIR\rtc_debug.sh

# Infer the previous version based on the existence of the scr.xml file
PREVIOUS_VERSION=2.x
case $UPGRADED_APP in
	jts)
		PREVIOUS_VERSION=3.0
		;;
	ccm | qm | rm)
		if [ "$APPLICATION_CONTEXT_ROOT" = "" ]; then
			if [ -f $OLD_APPLICATION_HOME/$UPGRADED_APP/scr.xml ]; then
				PREVIOUS_VERSION=3.0
				APPLICATION_CONTEXT_ROOT=$UPGRADED_APP
			else
				if [ $UPGRADED_APP = rm ]; then
					APPLICATION_CONTEXT_ROOT=rdm
				else
					APPLICATION_CONTEXT_ROOT=jazz
				fi
			fi
		fi
		if [ -f $OLD_APPLICATION_HOME/$APPLICATION_CONTEXT_ROOT/scr.xml ]; then
			PREVIOUS_VERSION=3.0
		fi
		;;
esac

# Set default OLD JTS Context root if necessary
if [ $UPGRADED_APP = rm ]; then
	if [ "$OLD_JTS_CONTEXT_ROOT" = "" ]; then
		if [ $PREVIOUS_VERSION = 2.x ]; then
			OLD_JTS_CONTEXT_ROOT=jazz
		else
			OLD_JTS_CONTEXT_ROOT=$JTS_CONTEXT_ROOT
		fi
	elif [ $OLD_JTS_CONTEXT_ROOT != $JTS_CONTEXT_ROOT ]; then
		messages INVALID_PARAM_VALUE -oldJTSContextRoot $OLD_JTS_CONTEXT_ROOT
		messages INVALID_PARAM_VALUE -newJTSContextRoot $JTS_CONTEXT_ROOT
		exit 3
	fi
fi

# Infer command file suffix depending on OS type
CMD_SUFFIX=sh
if [ "$OSTYPE" = "os400" ]; then
	#it ends with .qsh for v2.x and v3.0 on IBM i
	if [ $PREVIOUS_VERSION = 2.x ] || [ $PREVIOUS_VERSION = 3.0 ]; then
		CMD_SUFFIX=qsh
	fi
fi

# Old version parameters
if [ $UPGRADED_APP = jts ]; then
	if [ "$OLD_JTS_VERSION" != "" ]; then
		OLD_JTS_VERSION_ARG=oldJTSVersion=$OLD_JTS_VERSION
	fi
elif [ $UPGRADED_APP = ccm ]; then
	if [ "$OLD_APPLICATION_VERSION" != "" ]; then
		OLD_APPLICATION_VERSION_ARG=oldApplicationVersion=$OLD_APPLICATION_VERSION
	fi
fi

# Verify old application parameters
# -------------------------------------------
if [ $UPGRADED_APP != jts ]; then
	# Home directory: OLD_APPLICATION_HOME
	if [ "$OLD_APPLICATION_HOME" = "" ]; then
		messages MISSING_HOME_PARAM -oldApplicationHome
		exit 3
	fi
	if [ ! \( -d $OLD_APPLICATION_HOME \) ]; then
		messages DIR_NOT_FOUND $OLD_APPLICATION_HOME
		exit 3
	fi
	cd $OLD_APPLICATION_HOME/..

	# Context root: APPLICATION_CONTEXT_ROOT
	if [ "$APPLICATION_CONTEXT_ROOT" = "" ]; then
		APPLICATION_CONTEXT_ROOT=jazz
	fi

	# Do not validate the existence of repotools or teamserver.properties on RM 3.0.x since
	# they do not exist.
	if [ $UPGRADED_APP = rm ] && [ $PREVIOUS_VERSION != 2.x ]; then
		RM_30x=true
	else
		RM_30x=false
	fi
	# Repotools command: OLD_APPLICATION_CMD
	if [ $PREVIOUS_VERSION = 2.x ]; then
		CMD_NAME=repotools.$CMD_SUFFIX
	else
		CMD_NAME=repotools-$APPLICATION_CONTEXT_ROOT.$CMD_SUFFIX
	fi
	OLD_APP_PARENT_DIR=`pwd`
	if [ "$OSTYPE" != "os400" ] && [ $RM_30x = false ] && [ ! \( -f $OLD_APP_PARENT_DIR/$CMD_NAME \) ]; then
		if [ $PREVIOUS_VERSION = 2.x ]; then
			messages CMD_NOT_FOUND_2X_APP $OLD_APP_PARENT_DIR/$CMD_NAME $OLD_APPLICATION_HOME
		else
			messages CMD_NOT_FOUND_OLD_APP $OLD_APP_PARENT_DIR/$CMD_NAME $OLD_APPLICATION_HOME $APPLICATION_CONTEXT_ROOT
		fi
		exit 3
	fi

	# Old application teamserver.properties file
	if [ $UPGRADED_APP = rm ]; then
		OLD_APP_TSP_FILE=$OLD_APPLICATION_HOME/jazz/teamserver.properties
	else
		OLD_APP_TSP_FILE=$OLD_APPLICATION_HOME/$APPLICATION_CONTEXT_ROOT/teamserver.properties
	fi
	if [ $RM_30x = false ] && [ ! \( -f $OLD_APP_TSP_FILE \) ]; then
		if [ $PREVIOUS_VERSION = 2.x ]; then
			messages TSP_NOT_FOUND_2X_APP $OLD_APP_TSP_FILE $OLD_APPLICATION_HOME
		else
			messages TSP_NOT_FOUND_OLD_APP $OLD_APP_TSP_FILE $OLD_APPLICATION_HOME $APPLICATION_CONTEXT_ROOT
		fi
		exit 3
	fi

	# Home directory: NEW_APPLICATION_HOME
	if [ "$NEW_APPLICATION_HOME" = "" ]; then
		messages MISSING_HOME_PARAM -newApplicationHome
		exit 3
	fi
	if [ ! \( -d $NEW_APPLICATION_HOME \) ]; then
		messages DIR_NOT_FOUND $NEW_APPLICATION_HOME
		exit 3
	fi
	cd $NEW_APPLICATION_HOME/..

	if [ $UPGRADED_APP != rm ]; then
		# Repotools command: NEW_APP_CMD
		NEW_APP_PARENT_DIR=`pwd`
		NEW_APPLICATION_CMD=repotools-$APPLICATION_CONTEXT_ROOT.sh
		if [ ! \( -f $NEW_APP_PARENT_DIR/$NEW_APPLICATION_CMD \) ]; then
			messages CMD_NOT_FOUND_NEW_APP $NEW_APP_PARENT_DIR/$NEW_APPLICATION_CMD $NEW_APPLICATION_HOME $APPLICATION_CONTEXT_ROOT $PREVIOUS_VERSION
			exit 3
		fi

		# New application teamserver.properties file
		NEW_APP_TSP_FILE=$NEW_APPLICATION_HOME/$APPLICATION_CONTEXT_ROOT/teamserver.properties
		if [ ! \( -f $NEW_APP_TSP_FILE \) ]; then
			messages TSP_NOT_FOUND_NEW_APP $NEW_APP_TSP_FILE $NEW_APPLICATION_HOME $APPLICATION_CONTEXT_ROOT $PREVIOUS_VERSION
			exit 3
		fi
	fi
fi


# Infer and verify JTS parameters
# -------------------------------

# Context root: JTS_CONTEXT_ROOT (always defined)
if [ "$JTS_CONTEXT_ROOT" = "" ]; then
	JTS_CONTEXT_ROOT=jts
fi

# Old Home directory: OLD_JTS_HOME (not defined for 2.x upgrade or JTS already upgraded)
if [ $UPGRADED_APP = rm ]; then
	if [ "$OLD_JTS_HOME" = "" ]; then
		OLD_JTS_HOME=$OLD_APPLICATION_HOME
	fi
	if [ ! \( -d $OLD_JTS_HOME \) ]; then
		messages DIR_NOT_FOUND $OLD_JTS_HOME
		exit 3
	fi
	if [ "$JTS_PUBLIC_URL" != "" ]; then
		JTS_PUBLIC_URL_ARG="jtsURL=$JTS_PUBLIC_URL"
	fi
elif [ $UPGRADED_APP = jts ]; then
	if [ "$OLD_JTS_HOME" = "" ]; then
		messages MISSING_PARAM_VALUE -oldJTSHome
		exit 3
	else
		if [ ! \( -d $OLD_JTS_HOME \) ]; then
			messages DIR_NOT_FOUND $OLD_JTS_HOME
			exit 3
		fi
		cd $OLD_JTS_HOME/..

		# Check old repotools command
		OLD_JTS_PARENT_DIR=`pwd`
		if [ "$OSTYPE" != "os400" ]; then
			if [ ! \( -f $OLD_JTS_PARENT_DIR/repotools-$JTS_CONTEXT_ROOT.$CMD_SUFFIX \) ]; then
				messages CMD_NOT_FOUND_OLD_JTS $OLD_JTS_PARENT_DIR/repotools-$JTS_CONTEXT_ROOT.$CMD_SUFFIX $OLD_JTS_HOME $JTS_CONTEXT_ROOT
				exit 3
			fi
		fi

		# Check old JTS teamserver.properties file
		OLD_JTS_TSP_FILE=$OLD_JTS_HOME/$JTS_CONTEXT_ROOT/teamserver.properties
		if [ ! \( -f $OLD_JTS_TSP_FILE \) ]; then
			messages TSP_NOT_FOUND_OLD_JTS $OLD_JTS_TSP_FILE $OLD_JTS_HOME $JTS_CONTEXT_ROOT
			exit 3
		fi
	fi
fi

# New Home directory: NEW_JTS_HOME (always defined)
if [ "$NEW_JTS_HOME" = "" ]; then
	if [ $UPGRADED_APP = jts ]; then
		NEW_JTS_HOME="$START_DIR/conf"
	elif [ $UPGRADED_APP = rm ]; then
		NEW_JTS_HOME="$START_DIR/conf"
	else
		NEW_JTS_HOME=$NEW_APPLICATION_HOME
	fi
fi
if [ ! \( -d $NEW_JTS_HOME \) ]; then
	messages DIR_NOT_FOUND $NEW_JTS_HOME
	exit 3
fi
cd $NEW_JTS_HOME/..

# Repotools command: NEW_JTS_CMD (not defined for 3.0 upgrade)
NEW_JTS_PARENT_DIR=`pwd`
NEW_JTS_CMD=repotools-$JTS_CONTEXT_ROOT.sh
if [ ! \( -f $NEW_JTS_PARENT_DIR/$NEW_JTS_CMD \) ]; then
	messages CMD_NOT_FOUND_NEW_JTS $NEW_JTS_PARENT_DIR/$NEW_JTS_CMD $NEW_JTS_HOME $JTS_CONTEXT_ROOT
	exit 3
fi

# Check new JTS teamserver.properties file
NEW_JTS_TSP_FILE=$NEW_JTS_HOME/$JTS_CONTEXT_ROOT/teamserver.properties
if [ ! \( -f $NEW_JTS_TSP_FILE \) ]; then
	messages TSP_NOT_FOUND_NEW_JTS $NEW_JTS_TSP_FILE $NEW_JTS_HOME $JTS_CONTEXT_ROOT
	exit 3
fi

# For JTS upgrade the new application command is the new JTS repotools
if [ $UPGRADED_APP = jts ]; then
	NEW_APPLICATION_CMD=$NEW_JTS_CMD
	NEW_APP_PARENT_DIR=$NEW_JTS_PARENT_DIR
elif [ $UPGRADED_APP = rm ]; then
	# rm upgrade scripts are run from the 3.0.1 JTS repotools
	NEW_APPLICATION_CMD=$NEW_JTS_CMD
fi


# Verify Tomcat parameters
# ----------------------------------
if [ "$UPDATE_TOMCAT_FILES" != "" ]; then
	UPDATE_TOMCAT_FILES_ARG="updateTomcatFiles=$UPDATE_TOMCAT_FILES"
fi

# Old Tomcat home
if [ "$OLD_TOMCAT_HOME" != "" ]; then
	if [ ! \( -d $OLD_TOMCAT_HOME \) ]; then
		if [ ! \( -d $OLD_APP_PARENT_DIR/$OLD_TOMCAT_HOME \) ]; then
			if [ "`echo $OLD_TOMCAT_HOME | cut -d'/' -f1`" = "" ]; then
				messages DIR_NOT_FOUND $OLD_TOMCAT_HOME
			else
				messages DIR_NOT_FOUND $OLD_APP_PARENT_DIR/$OLD_TOMCAT_HOME
			fi
			exit 3
		fi
	fi
	if [ $UPGRADED_APP = rm ]; then
		OLD_TOMCAT_HOME_ARG="oldApplicationTomcatHome=$OLD_TOMCAT_HOME"
	else
		OLD_TOMCAT_HOME_ARG="oldTomcatHome=$OLD_TOMCAT_HOME"
	fi
fi
if [ $UPGRADED_APP = rm ]; then
	if [ "$OLD_JTS_TOMCAT_HOME" != "" ]; then
		if [ ! \( -d $OLD_JTS_TOMCAT_HOME \) ]; then
			if [ ! \( -d $OLD_APP_PARENT_DIR/$OLD_JTS_TOMCAT_HOME \) ]; then
				if [ "`echo $OLD_JTS_TOMCAT_HOME | cut -d'/' -f1`" = "" ]; then
					messages DIR_NOT_FOUND $OLD_JTS_TOMCAT_HOME
				else
					messages DIR_NOT_FOUND $OLD_APP_PARENT_DIR/$OLD_JTS_TOMCAT_HOME
				fi
				exit 3
			fi
		fi
		OLD_JTS_TOMCAT_HOME_ARG="oldJTSTomcatHome=$OLD_JTS_TOMCAT_HOME"
	fi
fi


# New Tomcat home
if [ "$NEW_TOMCAT_HOME" != "" ]; then
	if [ ! \( -d $NEW_TOMCAT_HOME \) ]; then
		if [ ! \( -d $NEW_APP_PARENT_DIR/$NEW_TOMCAT_HOME \) ]; then
			if [ "`echo $NEW_TOMCAT_HOME | cut -d'/' -f1`" = "" ]; then
				messages DIR_NOT_FOUND $NEW_TOMCAT_HOME
			else
				messages DIR_NOT_FOUND $NEW_APP_PARENT_DIR/$NEW_TOMCAT_HOME
			fi
			exit 3
		fi
	fi
	if [ $UPGRADED_APP = jts ]; then
		NEW_TOMCAT_HOME_ARG="newTomcatHome=$NEW_TOMCAT_HOME"
	else
		NEW_TOMCAT_HOME_ARG="newApplicationTomcatHome=$NEW_TOMCAT_HOME"
	fi
fi
if [ $UPGRADED_APP != jts ]; then
	if [ "$NEW_JTS_TOMCAT_HOME" != "" ]; then
		if [ ! \( -d $NEW_JTS_TOMCAT_HOME \) ]; then
			if [ ! \( -d $NEW_APP_PARENT_DIR/$NEW_JTS_TOMCAT_HOME \) ]; then
				if [ "`echo $NEW_JTS_TOMCAT_HOME | cut -d'/' -f1`" = "" ]; then
					messages DIR_NOT_FOUND $NEW_JTS_TOMCAT_HOME
				else
					messages DIR_NOT_FOUND $NEW_APP_PARENT_DIR/$NEW_JTS_TOMCAT_HOME
				fi
				exit 3
			fi
		fi
		NEW_JTS_TOMCAT_HOME_ARG="newJTSTomcatHome=$NEW_JTS_TOMCAT_HOME"
	fi
fi


# Other verifications
# -------------------

# Import parameter: IMPORT_FROM_FILE
if [ "$IMPORT_FROM_FILE" = "" ]; then
	if [ $PREVIOUS_VERSION = 2.x ]; then
		if [ $UPGRADED_APP = qm ] || [ $UPGRADED_APP = rm ]; then
			messages MISSING_IMPORT_FROM_FILE $UPGRADED_APP
			exit 3
		fi
	fi
fi

# Check data warehouse
UPGRADE_DATA_WAREHOUSE=false
if [ $PREVIOUS_VERSION != 2.x ] && [ $UPGRADED_APP = jts ]; then

	# Check if property is set in the new JTS TSP
	DATAWAREHOUSE_TSP_LINE=`grep "com.ibm.team.datawarehouse.db.jdbc.location" $OLD_JTS_TSP_FILE | grep -v "#com.ibm.team.datawarehouse.db.jdbc.location"`
	if [ "$DATAWAREHOUSE_TSP_LINE" != "" ]; then
		UPGRADE_DATA_WAREHOUSE=true
	fi
fi


# **************
# * Start exec *
# **************

messages BANNER_$UPGRADED_APP $PREVIOUS_VERSION


# *********************
# * First step switch *
# *********************

cd $START_DIR

# Init variables
INITIAL_STEP=0
REBUILD_TXT_INDICES="false"

# Set max steps to execute
case $UPGRADED_APP in
	jts)
		if [ $UPGRADE_DATA_WAREHOUSE = true ]; then
			MAX_STEPS=2
		else
			MAX_STEPS=1
		fi
		;;
	rm)
		if [ $PREVIOUS_VERSION = 3.0 ]; then
			MAX_STEPS=0
		else
			MAX_STEPS=1
		fi
		;;
	qm)
		if [ $PREVIOUS_VERSION = 3.0 ]; then
			MAX_STEPS=1
		else
			MAX_STEPS=4
		fi
		;;
	ccm)
		if [ $PREVIOUS_VERSION = 3.0 ]; then
			MAX_STEPS=1
		elif [ "$IMPORT_FROM_FILE" = "" ]; then
			MAX_STEPS=6
		else
			MAX_STEPS=4
		fi
		;;
esac

# Get last execution information
LAST_STEP=-1
LAST_RETURN=0
LOG_FILE=$SCRIPT_DIR/.${UPGRADED_APP}_app_upgrade.log
if [ -f $LOG_FILE ]; then
	LAST_STEP=`cut -f1 -d' ' $LOG_FILE`
	LAST_RETURN=`cut -f2 -d' ' $LOG_FILE`
	LAST_STEP_INFO=`cut -f3 -d' ' $LOG_FILE`
	if [ "$LAST_STEP_INFO" = "REBUILD_TXT_INDICES" ]; then
		MAX_STEPS=2
		REBUILD_TXT_INDICES="true"
	fi
	setFirstStep
else
	CURRENT_STEP=$INITIAL_STEP
fi


# **********************
# * Verify Public URIs *
# **********************
if [ $CURRENT_STEP -ne $INITIAL_STEP ]; then
	verify_all -clean
fi


# **************
# * Steps loop *
# **************
while [ $CURRENT_STEP -le $MAX_STEPS ]
do
	case $CURRENT_STEP in
		0)
			if [ "$OSTYPE" = "os400" ] && [ $PREVIOUS_VERSION = 2.x ]; then
				CURRENT_STEP_MSG=STEP_0_IBMi
				STEP_CMD="$NEW_JTS_CMD -dropTables $NO_PROMPT"
				runStep $NEW_JTS_PARENT_DIR
			fi
			
			CURRENT_STEP_MSG=STEP_0
			case $UPGRADED_APP in
				jts)
					CMD_ARGUMENTS="oldJTSHome=$OLD_JTS_HOME jtsContextRoot=$JTS_CONTEXT_ROOT $UPDATE_TOMCAT_FILES_ARG $OLD_TOMCAT_HOME_ARG newJTSHome=$NEW_JTS_HOME $NEW_TOMCAT_HOME_ARG $OLD_JTS_VERSION_ARG"
					;;
				rm)
					CMD_ARGUMENTS="oldApplicationHome=$OLD_APPLICATION_HOME applicationContextRoot=$APPLICATION_CONTEXT_ROOT oldJTSHome=$OLD_JTS_HOME oldJTSContextRoot=$OLD_JTS_CONTEXT_ROOT $UPDATE_TOMCAT_FILES_ARG $OLD_TOMCAT_HOME_ARG $OLD_JTS_TOMCAT_HOME_ARG newApplicationHome=$NEW_APPLICATION_HOME newJTSHome=$NEW_JTS_HOME newJTSContextRoot=$JTS_CONTEXT_ROOT $NEW_TOMCAT_HOME_ARG $NEW_JTS_TOMCAT_HOME_ARG $JTS_PUBLIC_URL_ARG $IGNORE_JTS_VERSION_CHECK"
					;;
				qm | ccm)
					CMD_ARGUMENTS="oldApplicationHome=$OLD_APPLICATION_HOME applicationContextRoot=$APPLICATION_CONTEXT_ROOT $UPDATE_TOMCAT_FILES_ARG $OLD_TOMCAT_HOME_ARG newApplicationHome=$NEW_APPLICATION_HOME newJTSHome=$NEW_JTS_HOME jtsContextRoot=$JTS_CONTEXT_ROOT $NEW_TOMCAT_HOME_ARG $NEW_JTS_TOMCAT_HOME_ARG $IGNORE_JTS_VERSION_CHECK $OLD_APPLICATION_VERSION_ARG"
					;;
			esac
			if [ $UPGRADED_APP = jts ] || [ $UPGRADED_APP = rm ]; then
				STEP_DIR=$NEW_JTS_PARENT_DIR
			else
				STEP_DIR=$NEW_APP_PARENT_DIR
			fi
			STEP_CMD="$NEW_APPLICATION_CMD -migration_${UPGRADED_APP}_updateConfigurationFiles $CMD_ARGUMENTS"
			runStep $STEP_DIR

			# If step was not skipped
			if [ $? -eq 1 ]; then
				#skip this step for IBM i 
				if [ "$OSTYPE" != "os400" ]; then
					# Verify teamserver.properties files
					if [ $ERROR_REPOTOOLS -eq 0 ]; then
						messages EDIT_TEAMSERVER_PROPERTIES JTS
						pause
						vi $NEW_JTS_TSP_FILE
						if [ $UPGRADED_APP != jts ] && [ $UPGRADED_APP != rm ]; then
							messages EDIT_TEAMSERVER_PROPERTIES $UPGRADED_APP
							pause
							vi $NEW_APP_TSP_FILE
						fi
					fi
				fi

				# Copy text indices if necessary
				if [ $UPGRADED_APP = ccm ] && [ $PREVIOUS_VERSION = 3.0 ]; then
					copy_text_indices
					# if text indices copy fails in any way, then force a text indices rebuild
					error_copy=$?
					if [ $error_copy -ne 0 ]; then
						MAX_STEPS=2
						REBUILD_TXT_INDICES="true"
					fi
				fi
			fi

			# Verify all teamserver.properties files
			verify_all
			;;

		1)
			if [ $UPGRADED_APP = rm ]; then
				CURRENT_STEP_MSG="STEP_1_IMPORT $PREVIOUS_VERSION"
				STEP_CMD="$NEW_APPLICATION_CMD -migration_importApplicationData fromFile=$IMPORT_FROM_FILE teamserver.properties=$NEW_JTS_HOME/$JTS_CONTEXT_ROOT/teamserver.properties $CREATE_JTS_TABLES $NO_PROMPT"
			elif [ "$IMPORT_FROM_FILE" = "" ]; then
				CURRENT_STEP_MSG=STEP_1_ADD
				STEP_CMD="$NEW_APPLICATION_CMD -addTables $NO_PROMPT"
			else
				CURRENT_STEP_MSG="STEP_1_IMPORT $PREVIOUS_VERSION"
				STEP_CMD="$NEW_APPLICATION_CMD -import fromFile=$IMPORT_FROM_FILE $NO_PROMPT"
			fi
			if [ $UPGRADED_APP = jts ] || [ $UPGRADED_APP = rm ]; then
				STEP_DIR=$NEW_JTS_PARENT_DIR
			else
				STEP_DIR=$NEW_APP_PARENT_DIR
			fi
			runStep $STEP_DIR
			;;

		2)
			if [ $UPGRADED_APP = jts ]; then
				# %UPGRADE_DATA_WAREHOUSE% is true in this case...
				CURRENT_STEP_MSG=STEP_2_UPGRADE_DATA_WAREHOUSE
				STEP_CMD="$NEW_JTS_CMD -createWarehouse teamserver.properties=$NEW_JTS_TSP_FILE $NO_PROMPT"
				runStep $NEW_JTS_PARENT_DIR
			elif [ $REBUILD_TXT_INDICES = "true" ]; then
				CURRENT_STEP_MSG=STEP_5
				STEP_CMD="$NEW_APPLICATION_CMD -rebuildTextIndices"
				runStep $NEW_APP_PARENT_DIR
			else
				CURRENT_STEP_MSG=STEP_2_EXPORT_USER_INFO
				STEP_CMD="$NEW_APPLICATION_CMD -migration_exportJTSData toFile=jtsData.tar $OVERWRITE"
				runStep $NEW_APP_PARENT_DIR
			fi
			;;

		3)
			CURRENT_STEP_MSG=STEP_3_IMPORT_USER_INFO
			if [ "$OSTYPE" = "os400" ] && [ $PREVIOUS_VERSION = 2.x ]; then
				CREATE_JTS_TABLES=createTables
			fi
			
			STEP_CMD="$NEW_JTS_CMD -migration_importJTSData fromFile=$NEW_APP_PARENT_DIR/jtsData.tar createMappingFile=$NEW_APP_PARENT_DIR/appli-mapping.txt $CREATE_JTS_TABLES $OVERWRITE_ARG $OVERWRITE"
			runStep $NEW_JTS_PARENT_DIR
			;;

		4)
			CURRENT_STEP_MSG=STEP_4
			STEP_CMD="$NEW_APPLICATION_CMD -migration_importUserMapping fromMappingFile=appli-mapping.txt"
			runStep $NEW_APP_PARENT_DIR
			;;

		# =========================================
		# === Post-Migration repotools commands ===
		# =========================================
		5)
			CURRENT_STEP_MSG=STEP_5
			STEP_CMD="$NEW_APPLICATION_CMD -rebuildTextIndices"
			runStep $NEW_APP_PARENT_DIR
			;;

		6)
			CURRENT_STEP_MSG=STEP_6
			STEP_CMD="$NEW_APPLICATION_CMD -reindex"
			runStep $NEW_APP_PARENT_DIR
			;;

	esac

	# Store information to restart properly
	if [ $CURRENT_STEP -eq $MAX_STEPS ] || [ $REBUILD_TXT_INDICES = "false" ]; then
		echo "$CURRENT_STEP $ERROR_REPOTOOLS" > $LOG_FILE
	else
		echo "$CURRENT_STEP $ERROR_REPOTOOLS REBUILD_TXT_INDICES" > $LOG_FILE
	fi
	
	# Verify that command was run successfully
	if [ $ERROR_REPOTOOLS -eq 0 ]; then
		if [ $CURRENT_STEP -eq $MAX_STEPS ]; then
			if [ $UPGRADED_APP = jts ]; then
				messages OVERALL_MIGRATION_SUCCESS_JTS $UPGRADED_APP
			elif [ $UPGRADED_APP = rm ]; then
				if [ $PREVIOUS_VERSION = 2.x ]; then
					messages OVERALL_MIGRATION_SUCCESS_RM $UPGRADED_APP
				fi
			else 
				messages OVERALL_MIGRATION_SUCCESS_CCM_QM $UPGRADED_APP
			fi
		fi	
		CURRENT_STEP=`expr $CURRENT_STEP + 1`
	else
		messages OVERALL_MIGRATION_FAILED $UPGRADED_APP $CURRENT_STEP
		exit $ERROR_REPOTOOLS
	fi
done
